/***
 * @author James Padolsey, José Luis López Pino
 * @from 	https://forja.rediris.es/scm/?group_id=736
 * 			http://github.com/jamespadolsey/PhotoWall
 */

var createElement = (function(){
	
	return function(type, attrs) {
		
		var el = document.createElement(type),
			attr, css, event, events;
		
		if (attrs) {
			
			css = attrs.css;
			
			if (css) {
				for (var prop in css) {
					if (!css.hasOwnProperty || css.hasOwnProperty(prop)) {
						try{
							el.style[prop] = typeof css[prop] === 'string' ? css[prop] : css[prop] + 'px';
						} catch(e) { /* Invalid props */ }
					}
				}
			}
			
		}
		
		for (attr in attrs) {
			if (!attrs.hasOwnProperty || attrs.hasOwnProperty(attr)) {
				if (attr !== 'css') {
					el[attr] = attrs[attr];
				}
			}
		}
		
		return el;
	};
	
})();


PhotoWall = (function() {
	
	var defaults = {
		width: 800,
		height: 300,
		topMargin: 100,
		inject: function() {}
	};
	
	function merge(target, source) {
		
		if ( typeof target !== 'object' ) {
			target = {};
		}
		
		for (var property in source) {
		
			if ( source.hasOwnProperty(property) ) {
				
				var sourceProperty = source[ property ];
				
				if ( typeof sourceProperty === 'object' ) {
					target[ property ] = merge( target[ property ], sourceProperty );
					continue;
				}
				
				target[ property ] = sourceProperty;
				
			}
		
		}
		
		for (var a = 2, l = arguments.length; a < l; a++) {
			merge(target, arguments[a]);
		}
		
		return target;
	}
	
	function fill(arr, start, end, fn) {
		
		// Fills a portion (start>end) of an array
		// with fn, or if fn's a function, fn();
		
		var isFn = typeof fn === 'function';
		
		for(var i = start; i < end; ++i) {
			arr[i] = isFn ? fn() : fn;
		}
		
		return arr;
	
	}
	
	function sortHighest(a,b) {
		
		// Sort so points with lowest Y values are pushed
		// to front.
		
		if ( a.y === b.y ) {
			// Both are equal, go for the left-most one
			return a.x - b.x;
		}
		
		return a.y - b.y// + (a.x - b.x);
	}
	
	function PhotoWall(ops, imgs) {
		
		ops = merge(defaults, ops);
		
		var container = createElement('div', {
				styles: {
					width: typeof ops.width === 'string' ? ops.width : ops.width + 'px',
					height: typeof ops.height === 'string' ? ops.height : ops.height + 'px',
					position: 'relative'
				},
				id: 'photowall'
				
			});
		
		container.setAttribute('class', 'ceebox mixed {videoWidth:200}');
		
		width = parseFloat(ops.width),
		height = parseFloat(ops.height),
		
		highestPoints = [{x:0,y:0}],
		occupied = [];
			
		// Fill with empty arrays x
		fill(occupied, 0, height, function(){
			return fill([], 0, width, false);
		});
		
		function getNewPosition( whRatio, maxW, minW, maxH, minH ) {
			//~ console.log( "Buscando posicion MW: " + maxW + " mW " + minW + " MH " + maxH + " mH " + minH);
			var highestPoint = highestPoints.shift();
			
			if ( !highestPoint ) {
				return null;
			}
			
			var x = highestPoint.x,
				y = highestPoint.y,
				wx = x,
				hy = y,
				position = {};
			
			while (
				occupied[y+1] &&
				occupied[y+1][++wx] === false &&
				//~ wx - x < ops.maxImgWidth && // Max width check
				wx - x < maxW && // Max width check
				//~ Math.round((wx-x) / whRatio) < Math.min(ops.maxImgHeight, height-y) // Max height check
				Math.round((wx-x) / whRatio) < Math.min(maxH, height-y) // Max height check
			) {}
			
			position = {
				top : y,
				left: x,
				width: wx - x,
				height: Math.round((wx-x) / whRatio)
			};
			
			//~ if (position.width < ops.minImgWidth || position.height < ops.minImgHeight) {
			if (position.width < minW || position.height < minH) {
				return getNewPosition( whRatio, maxW, minW, maxH, minH );
			}
			
			var hy = position.height + y;
			
			for( var i = y; i < hy; ++i) {
				if (occupied[i]) fill(
					occupied[i],
					x,
					wx,
					true
				);
			}
			
			//if (!window.l) {//console.log(occupied.toString());window.l = true;}
			
			highestPoints.push({
				x: wx < width ? wx : 0,
				y: wx < width ? y : hy
			});
			
			if ( hy < height ) {
				highestPoints.push({
					x: x,
					y: hy
				});
				highestPoints.push({
					x: wx,
					y: hy
				});
			}
			
			highestPoints.sort(sortHighest);
			
			return position;
			
		}
		
		var imgHeight = 0,
			imgWidth = 0,
			loaded = 0,
			img;
		
		var i = -1,
			position,
			cur,
			curImg;
		
		function next() {
			return imgs[i + 1 >= imgs.length ? i = 0 : ++i].url;
		}
		
		while (
			cur = next(),
			//~ console.log( "Buscando posición para " + imgs[i].url),
			position = getNewPosition(
				imgs[i].lWidth / imgs[i].lHeight, imgs[i].maxW, imgs[i].minW, imgs[i].maxH, imgs[i].minH
			)
		) {
			//console.log( "Uso " + imgs[i].url + "\tUsado?: " + imgs[i].used);
			
			if(imgs[i].used == 0){
				imgs[i].used = 1;
				
				position.top = position.top + ops.topMargin;
				curImg = createElement('img', {
				alt: imgs[i].name,
					src: cur,
					css: merge({
						position: 'absolute'
					}, position)
				});
				
				newlink = document.createElement('a');
				newlink.setAttribute('href', imgs[i].link);
				newlink.setAttribute('title', imgs[i].name);
				newlink.appendChild(curImg);
				
				//console.log(imgs[i].lWidth / imgs[i].lHeight);
				container.appendChild(newlink);
			}
		}
		
		var jscode=document.createElement('script');
		jscode.setAttribute('type','text/javascript');
		var text=document.createTextNode('$(".ceebox").ceebox();');
		jscode.appendChild(text);
		container.appendChild(jscode);
		
		ops.inject.call(container);
		
	};
	
	PhotoWall.defaults = defaults;
	
	return PhotoWall;
	
})();

